package com.dahua.openpicdecode.lcopensdk;

import com.dahua.openpicdecode.utils.ImageUtils;
import com.sun.jna.ptr.IntByReference;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * 乐橙解密相关工具类
 * 		DecSuccess = 0,   解密成功
 *		DecNotWhole = 1, 数据不完整
 *		DecKeyError = 2,  密钥错误
 *		DecUnEncrypt = 3, 图片未加密
 *		DecUnsupportEncryption = 4,  不支持的加密方式
 *		DecBufNotEnough = 5,  缓冲区长度不够
 *		DecInternalError = 99,  未知错误
 */

public class LechangeEncryptUtil {
	
	private static Logger logger= LoggerFactory.getLogger(LechangeEncryptUtil.class);
	
	private final static int DECRYPT_SUCCESS_CODE = 0;
	
	/**
	 * 图片解密
	 * @param imgUrl 待解密图片url
	 * @param secretKey 解密秘钥
	 * @return
	 */
	public static byte[] imageEncrypt(String imgUrl, String secretKey, String token, String sn) {
		if(StringUtils.isNotEmpty(imgUrl)) {
			//获取加密图片byte字节
			byte[] inputStr;
			try {
				inputStr = ImageUtils.generateByte(imgUrl);
				if(inputStr != null) {
					int len = inputStr.length;
					byte[] outputByte = new byte[len * 2];
					IntByReference  destBufLenOut = new IntByReference(len * 2);
					int returnStr;
					if(StringUtils.isEmpty(sn)){
						returnStr = LCOpenSdkSingleton.getInstance().decryptPic(imgUrl, secretKey, outputByte, destBufLenOut, token);
					}else {
						returnStr = LCOpenSdkSingleton.getInstance().decryptPicTcm(imgUrl, secretKey, sn, outputByte, destBufLenOut, token);
					}
					int outLen = destBufLenOut.getValue();
					logger.info("乐橙加密图片解密  解密响应: {}, imgUrl: {}, outLen： {}", returnStr, imgUrl, outLen);
					if(outLen > 0 && returnStr == DECRYPT_SUCCESS_CODE) {
						byte[] bytes = new byte[outLen];
						System.arraycopy(outputByte, 0, bytes, 0, outLen);
						return bytes;
					}
				}
			} catch (Exception e) {
				logger.error("乐橙加密图片解密发生异常", e);
			}
		}
		return null;
	}
}
