package com.dahua.openpicdecode.utils;

import javax.imageio.ImageIO;
import java.awt.*;
import java.awt.color.ColorSpace;
import java.awt.image.*;
import java.io.*;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLEncoder;

public class ImageUtils {

	/**
	 * 缩放图像
	 * 
	 * @param srcImageFile 源图像文件地址
	 * @param result 缩放后的图像地址
	 * @param scale 缩放大小
	 * @param flag 图片选择:true 缩略图; false 比例图;
	 * @param identifying 标识 1-缩略图，2-比例图
	 */
	public static void scale(String srcImageFile, String result, int scale,	boolean flag) {
		try {
			BufferedImage src = ImageIO.read(new File(srcImageFile)); // 读入文件

			int width = src.getWidth(); // 得到源图宽
			int height = src.getHeight(); // 得到源图长
			if (flag){
				if (width-height>0) {
					height = (int)(height*((float)scale/width));
					width = scale;
				}else{
					width = (int)(width*((float)scale/height));
					height = scale ;
				}
			}else{
				height = (int)(height*((float)scale/width));
				width = scale;
			}
			
			Image image = src.getScaledInstance(width, height, Image.SCALE_DEFAULT);
			BufferedImage tag = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
			Graphics g = tag.getGraphics();
			g.drawImage(image, 0, 0, null); // 绘制缩小后的图
			g.dispose();
			ImageIO.write(tag, "JPEG", new File(result));// 输出到文件流
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	public static byte[] scale(String imageUrl,int height,int width) {
		ByteArrayOutputStream baos = null;
		HttpURLConnection connection = null;
		DataInputStream in = null;
		try {
			if(imageUrl == null || imageUrl.isEmpty()) {
				return null;
			}
			URL url = new URL(imageUrl);
			/* 此为联系获得网络资源的固定格式用法，以便后面的in变量获得url截取网络资源的输入流 */
			connection = (HttpURLConnection) url.openConnection();
			connection.setConnectTimeout(3000);
			connection.setReadTimeout(3000);
			in = new DataInputStream(connection.getInputStream());
			BufferedImage src = ImageIO.read(in);
			if(src == null) {
			    return null;
			}
			Image image = src.getScaledInstance(width, height, Image.SCALE_DEFAULT);
			BufferedImage tag = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
			Graphics g = tag.getGraphics();
			g.drawImage(image, 0, 0, null); // 绘制缩小后的图
			g.dispose();
			baos = new ByteArrayOutputStream();
			ImageIO.write(tag, "JPEG", baos);// 输出到文件流
			return baos.toByteArray();
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			try {
				if(baos != null) {
					baos.close();
				}
				if(in != null) {
				    in.close();
				}
				if(connection != null) {
				    connection.disconnect();
				}
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		return null;
	}

	/**
	 * 图像切割
	 * 
	 * @param srcImageFile 源图像地址
	 * @param descDir 切片目标文件夹
	 * @param destWidth 目标切片宽度
	 * @param destHeight 目标切片高度
	 */
	public static void cut(String srcImageFile, String descDir, int destWidth,
			int destHeight) {
		try {
			Image img;
			ImageFilter cropFilter;
			// 读取源图像
			BufferedImage bi = ImageIO.read(new File(srcImageFile));
			int srcWidth = bi.getHeight(); // 源图宽度
			int srcHeight = bi.getWidth(); // 源图高度
			if (srcWidth > destWidth && srcHeight > destHeight) {
				Image image = bi.getScaledInstance(srcWidth, srcHeight, Image.SCALE_DEFAULT);
				destWidth = 200; // 切片宽度
				destHeight = 150; // 切片高度
				int cols = 0; // 切片横向数量
				int rows = 0; // 切片纵向数量
				// 计算切片的横向和纵向数量
				if (srcWidth % destWidth == 0) {
					cols = srcWidth / destWidth;
				} else {
					cols = (int) Math.floor(srcWidth / destWidth) + 1;
				}
				if (srcHeight % destHeight == 0) {
					rows = srcHeight / destHeight;
				} else {
					rows = (int) Math.floor(srcHeight / destHeight) + 1;
				}
				// 循环建立切片
				// 改进的想法:是否可用多线程加快切割速度
				for (int i = 0; i < rows; i++) {
					for (int j = 0; j < cols; j++) {
						// 四个参数分别为图像起点坐标和宽高
						// 即: CropImageFilter(int x,int y,int width,int height)
						cropFilter = new CropImageFilter(j * 200, i * 150, destWidth, destHeight);
						img = Toolkit.getDefaultToolkit().createImage(
								new FilteredImageSource(image.getSource(), cropFilter));
						BufferedImage tag = new BufferedImage(destWidth, destHeight, BufferedImage.TYPE_INT_RGB);
						Graphics g = tag.getGraphics();
						g.drawImage(img, 0, 0, null); // 绘制缩小后的图
						g.dispose();
						// 输出为文件
						ImageIO.write(tag, "JPEG", new File(descDir
								+ "pre_map_" + i + "_" + j + ".jpg"));
					}
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * 图像类型转换 GIF->JPG GIF->PNG PNG->JPG PNG->GIF(X)
	 */
	public static void convert(String source, String result) {
		try {
			File f = new File(source);
			f.canRead();
			f.canWrite();
			BufferedImage src = ImageIO.read(f);
			ImageIO.write(src, "JPG", new File(result));
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	/**
	 * 彩色转为黑白
	 * 
	 * @param source
	 * @param result
	 */
	public static void gray(String source, String result) {
		try {
			BufferedImage src = ImageIO.read(new File(source));
			ColorSpace cs = ColorSpace.getInstance(ColorSpace.CS_GRAY);
			ColorConvertOp op = new ColorConvertOp(cs, null);
			src = op.filter(src, null);
			ImageIO.write(src, "JPEG", new File(result));
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	
	/**
	 * 缩放图像
	 * 
	 * @param srcImageFile 源图像文件地址
	 * @param result 缩放后的图像地址
	 * @param scale 缩放大小
	 * @param flag 图片选择:true 缩略图; false 比例图;
	 * @param identifying 标识 1-缩略图，2-比例图
	 */
	public static BufferedImage compressImage(InputStream input , int scale,boolean flag) {
		try {
			BufferedImage src = ImageIO.read(input); // 读入文件
            if (src == null) {
            	return null;
			}
			int width = src.getWidth(); // 得到源图宽
			int height = src.getHeight(); // 得到源图长
			if (flag){
				if (width-height>0) {
					height = (int)(height*((float)scale/width));
					width = scale;
				}else{
					width = (int)(width*((float)scale/height));
					height = scale ;
				}
			}else{
				height = (int)(height*((float)scale/width));
				width = scale;
			}
			
			Image image = src.getScaledInstance(width, height, Image.SCALE_DEFAULT);
			BufferedImage tag = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
			Graphics g = tag.getGraphics();
			g.drawImage(image, 0, 0, null); // 绘制缩小后的图
			g.dispose();
//			ImageIO.write(tag, "JPEG", new File(result));// 输出到文件流
			return tag;
		} catch (IOException e) {
			e.printStackTrace();
		}
		return null;
	}
	
	public static byte[] convertImageToBytes(String fileUrl) {
		HttpURLConnection connection = null;
		byte[] bytes = null;
		InputStream in = null;
		ByteArrayOutputStream os = null;
    	try{
    		String prefixURL = fileUrl.substring(0,fileUrl.lastIndexOf("/")+1);
			String suffixURL = fileUrl.substring(fileUrl.lastIndexOf("/")+1);
			
			URL url = new URL(prefixURL+URLEncoder.encode(suffixURL, "UTF-8"));
			 //此为联系获得网络资源的固定格式用法，以便后面的in变量获得url截取网络资源的输入流 
			connection = (HttpURLConnection) url.openConnection();
			connection.setConnectTimeout(3000);
			connection.setReadTimeout(3000);
			in = connection.getInputStream();
			int readLen;
			int fileLength = connection.getContentLength();
			byte[] buf = new byte[fileLength];
			os = new ByteArrayOutputStream();
			while ((readLen = in.read(buf)) != -1 && --fileLength > 0) {
				os.write(buf, 0, readLen);
			}
            byte[] buffer = os.toByteArray();  
            return buffer;
        }catch(Exception e) {
        	e.printStackTrace();
        }finally{
        	try {
        		if(in != null){
        			in.close();
        		}
        		if(os != null){
        			os.flush();
        			os.close();
        			os = null;
        		}
        		if(connection != null){
        			connection.disconnect();
        		}
			} catch (IOException e) {
			}
        }
    	return bytes;
	}
	
	public static byte[] generateByte(String imgUrl) throws Exception {
		if(imgUrl == null || imgUrl.isEmpty()) {
			return null;
		}
		HttpURLConnection connection = null;
		DataInputStream in = null;
		ByteArrayOutputStream outByte = null;
		URL url = null;
		try {
			url = new URL(imgUrl);
			/* 此为联系获得网络资源的固定格式用法，以便后面的in变量获得url截取网络资源的输入流 */
	        connection = (HttpURLConnection) url.openConnection();
	        connection.setConnectTimeout(3000);
	        connection.setReadTimeout(3000);
	        in = new DataInputStream(connection.getInputStream());
	        outByte = new ByteArrayOutputStream();
	        /* 将参数savePath，即将截取的图片的存储在本地地址赋值给out输出流所指定的地址 */  
	        byte[] buffer = new byte[4096];  
	        int count = 0;
	        /* 将输入流以字节的形式读取并写入buffer中 */  
	        while ((count = in.read(buffer)) > 0){  
	        	outByte.write(buffer, 0, count);  
	        }
	        outByte.flush();
	        return outByte.toByteArray();
		} catch (Exception e) {
			throw e;
		}finally {
			 try {
			     if(in != null) {
			         in.close();
			     }
				 if(connection != null) {
				     connection.disconnect();
				 }
				 if(outByte != null) {
				     outByte.close();
				 }
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}

}
